@@ -74,7 +74,7 @@ bool ZrtpCallbackWrapper::srtpSecretsReady(SrtpSecret_t* secrets, EnableSecurity
 
     bool retval = (c_callbacks->zrtp_srtpSecretsReady(zrtpCtx, cs, (int32_t)part) == 0) ? false : true ;
 
-    delete cs->sas;
+    delete[] cs->sas;
     delete cs;
 
     return retval;
@@ -127,47 +127,25 @@ void ZrtpCallbackWrapper::synchLeave()
     c_callbacks->zrtp_synchLeave(zrtpCtx);
 }
 
-void ZrtpCallbackWrapper::zrtpAskEnrollment(std::string info)
+void ZrtpCallbackWrapper::zrtpAskEnrollment(GnuZrtpCodes::InfoEnrollment info)
 {
-    char* cc = new char [info.size()+1];
+    c_callbacks->zrtp_zrtpAskEnrollment(zrtpCtx, (zrtp_InfoEnrollment)info);
+}
 
-    strcpy(cc, info.c_str());
-    c_callbacks->zrtp_zrtpAskEnrollment(zrtpCtx, cc);
-
-    delete[] cc;
+void ZrtpCallbackWrapper::zrtpInformEnrollment(GnuZrtpCodes::InfoEnrollment info)
+{
+    c_callbacks->zrtp_zrtpInformEnrollment(zrtpCtx, (zrtp_InfoEnrollment)info);
 
 }
 
-void ZrtpCallbackWrapper::zrtpInformEnrollment(std::string info)
+void ZrtpCallbackWrapper::signSAS(uint8_t* sasHash)
 {
-    char* cc = new char [info.size()+1];
-
-    strcpy(cc, info.c_str());
-    c_callbacks->zrtp_zrtpInformEnrollment(zrtpCtx, cc);
-
-    delete[] cc;
-
+    c_callbacks->zrtp_signSAS(zrtpCtx, sasHash);
 }
 
-void ZrtpCallbackWrapper::signSAS(std::string sas)
+bool ZrtpCallbackWrapper::checkSASSignature(uint8_t* sasHash)
 {
-    char* cc = new char [sas.size()+1];
-
-    strcpy(cc, sas.c_str());
-    c_callbacks->zrtp_signSAS(zrtpCtx, cc);
-
-    delete[] cc;
-
-}
-
-bool ZrtpCallbackWrapper::checkSASSignature(std::string sas )
-{
-    char* cc = new char [sas.size()+1];
-
-    strcpy(cc, sas.c_str());
-    bool retval = (c_callbacks->zrtp_checkSASSignature(zrtpCtx, cc) == 0) ? false : true;
-
-    delete[] cc;
+    bool retval = (c_callbacks->zrtp_checkSASSignature(zrtpCtx, sasHash) == 0) ? false : true;
 
     return retval;
 }
