@@ -38,6 +38,8 @@
 #include <libzrtpcpp/ZrtpPacketErrorAck.h>
 #include <libzrtpcpp/ZrtpPacketPing.h>
 #include <libzrtpcpp/ZrtpPacketPingAck.h>
+#include <libzrtpcpp/ZrtpPacketSASrelay.h>
+#include <libzrtpcpp/ZrtpPacketRelayAck.h>
 #include <libzrtpcpp/ZrtpCallback.h>
 #include <libzrtpcpp/ZIDRecord.h>
 
@@ -93,7 +95,7 @@ class __EXPORT ZRtp {
      * engine.
      */
     ZRtp(uint8_t* myZid, ZrtpCallback* cb, std::string id,
-         ZrtpConfigure* config);
+         ZrtpConfigure* config, bool mitmm= false, bool sasSignSupport= false);
 
     /**
      * Destructor cleans up.
@@ -173,19 +175,6 @@ class __EXPORT ZRtp {
      */
     void setAuxSecret(uint8_t* data, int32_t length);
 
-    /**
-     * Set the PBX secret.
-     *
-     * Use this method to set the PBX secret data. Refer to ZRTP
-     * specification, chapter 4.3 ff and 7.3
-     *
-     * @param data
-     *     Points to the other PBX data.
-     * @param length
-     *     The length in bytes of the data.
-     */
-    void setPbxSecret(uint8_t* data, int32_t length);
-
     /**
      * Check current state of the ZRTP state engine
      *
@@ -210,18 +199,37 @@ class __EXPORT ZRtp {
      */
     void resetSASVerified();
 
-   /**
-    * Get the ZRTP Hello Hash data.
-    *
-    * Use this method to get the ZRTP Hello Hash data. The method
-    * returns the data as a string containing the ZRTP protocol version and
-    * hex-digits. Refer to ZRTP specification, chapter 8.
-    *
-    * @return
-    *    a std:string containing the Hello hash value as hex-digits. The
-    *    hello hash is available immediately after class instantiation.
-    */
-   std::string getHelloHash();
+    /**
+     * Get the ZRTP Hello Hash data.
+     *
+     * Use this method to get the ZRTP Hello Hash data. The method
+     * returns the data as a string containing the ZRTP protocol version and
+     * hex-digits.
+     *
+     * Refer to ZRTP specification, chapter 8.
+     *
+     * @return
+     *    a std:string containing the Hello hash value as hex-digits. The
+     *    hello hash is available immediately after class instantiation.
+     */
+    std::string getHelloHash();
+
+    /**
+     * Get the peer's ZRTP Hello Hash data.
+     *
+     * Use this method to get the peer's ZRTP Hello Hash data. The method
+     * returns the data as a string containing the ZRTP protocol version and
+     * hex-digits.
+     *
+     * The peer's hello hash is available only after ZRTP received a hello. If
+     * no data is available the function returns an empty string.
+     *
+     * Refer to ZRTP specification, chapter 8.
+     *
+     * @return
+     *    a std:string containing the Hello version and the hello hash as hex digits.
+     */
+    std::string getPeerHelloHash();
 
     /**
      * Get Multi-stream parameters.
@@ -297,22 +305,73 @@ class __EXPORT ZRtp {
     void acceptEnrollment(bool accepted);
 
     /**
-     * Enable PBX enrollment
-     *
-     * The application calls this method to allow or disallow PBX enrollment.
-     * If the applications allows PBX enrollment then the ZRTP implementation
-     * honors the PBX enrollment flag in Confirm packets. Refer to chapter 7.3
-     * for further details of PBX enrollment.
-     *
-     * @param yesNo
-     *    If set to true then ZRTP honors the PBX enrollment flag in Commit
-     *    packets and calls the appropriate user callback methods. If
-     *    the parameter is set to false ZRTP ignores the PBX enrollment flags.
+     * Check the state of the enrollment mode.
+     * 
+     * If true then we will set the enrollment flag (E) in the confirm
+     * packets and perform the enrollment actions. A MitM (PBX) enrollment service
+     * started this ZRTP session. Can be set to true only if mitmMode is also true.
+     * 
+     * @return status of the enrollmentMode flag.
      */
-    void setPBXEnrollment(bool yesNo);
+    bool isEnrollmentMode();
 
     /**
-     * Set signature data
+     * Set the state of the enrollment mode.
+     * 
+     * If true then we will set the enrollment flag (E) in the confirm
+     * packets and perform the enrollment actions. A MitM (PBX) enrollment 
+     * service must sets this mode to true. 
+     * 
+     * Can be set to true only if mitmMode is also true. 
+     * 
+     * @param enrollmentMode defines the new state of the enrollmentMode flag
+     */
+    void setEnrollmentMode(bool enrollmentMode);
+
+    /**
+     * Check if a peer's cache entry has a vaild MitM key.
+     *
+     * If true then the other peer ha a valid MtiM key, i.e. the peer has performed
+     * the enrollment procedure. A PBX ZRTP Back-2-Back application can use this function
+     * to check which of the peers is enrolled.
+     *
+     * @return True if the other peer has a valid Mitm key (is enrolled).
+     */
+    bool isPeerEnrolled();
+
+    /**
+     * Send the SAS relay packet.
+     * 
+     * The method creates and sends a SAS relay packet according to the ZRTP
+     * specifications. Usually only a MitM capable user agent (PBX) uses this
+     * function.
+     * 
+     * @param sh the full SAS hash value, 32 bytes
+     * @param render the SAS rendering algorithm
+     */
+    bool sendSASRelayPacket(uint8_t* sh, std::string render);
+
+    /**
+     * Get the commited SAS rendering algorithm for this ZRTP session.
+     * 
+     * @return the commited SAS rendering algorithm
+     */
+    std::string getSasType();
+ 
+    /**
+     * Get the computed SAS hash for this ZRTP session.
+     *
+     * A PBX ZRTP back-to-Back function uses this function to get the SAS
+     * hash of an enrolled client to construct the SAS relay packet for
+     * the other client.
+     *
+     * @return a pointer to the byte array that contains the full
+     *         SAS hash.
+     */
+    uint8_t* getSasHash();
+
+    /**
+     * Set signature data.
      *
      * This functions stores signature data and transmitts it during ZRTP
      * processing to the other party as part of the Confirm packets. Refer to
@@ -333,32 +392,32 @@ class __EXPORT ZRtp {
     bool setSignatureData(uint8_t* data, int32_t length);
 
     /**
-     * Get signature data
+     * Get signature data.
      *
-     * This functions returns signature data that was receivied during ZRTP
-     * processing. Refer to chapters 5.7 and 7.2.
+     * This functions returns a pointer to the signature data that was receivied
+     * during ZRTP processing. Refer to chapters 5.7 and 7.2.
+     *
+     * The returned pointer points to volatile data that is valid only during the
+     * <code>checkSASSignature()</code> callback funtion. The application must copy
+     * the signature data if it will be used after the callback function returns.
      *
      * The signature data can be retrieved after ZRTP enters secure state.
      * <code>start()</code>.
      *
-     * @param data
-     *    Pointer to a data buffer. This buffer must be large enough to
-     *    hold the signature data. Refer to <code>getSignatureLength()</code>
-     *    to get the length of the received signature data.
      * @return
-     *    Number of bytes copied into the data buffer
+     *    Pointer to signature data.
      */
-    int32_t getSignatureData(uint8_t* data);
+    const uint8_t* getSignatureData();
 
     /**
-     * Get length of signature data
+     * Get length of signature data in number of bytes.
      *
      * This functions returns the length of signature data that was receivied
      * during ZRTP processing. Refer to chapters 5.7 and 7.2.
      *
      * @return
      *    Length in bytes of the received signature data. The method returns
-     *    zero if no signature data avilable.
+     *    zero if no signature data is avilable.
      */
     int32_t getSignatureLength();
 
@@ -390,7 +449,7 @@ class __EXPORT ZRtp {
       *    Number of bytes copied into the data buffer - must be equivalent
       *    to 96 bit, usually 12 bytes.
       */
-     int32_t getZid(uint8_t* data);
+     int32_t getPeerZid(uint8_t* data);
 
 private:
      friend class ZrtpStateClass;
@@ -516,6 +575,9 @@ private:
     uint8_t H3[IMPL_MAX_DIGEST_LENGTH];
     uint8_t helloHash[IMPL_MAX_DIGEST_LENGTH];
 
+    uint8_t peerHelloHash[IMPL_MAX_DIGEST_LENGTH];
+    uint8_t peerHelloVersion[ZRTP_WORD_SIZE + 1];   // +1 for nul byte
+
     // We get the peer's H? from the message where length is defined as 8 words
     uint8_t peerH0[8*ZRTP_WORD_SIZE];
     uint8_t peerH1[8*ZRTP_WORD_SIZE];
@@ -526,6 +588,7 @@ private:
      * The SHA256 hash over selected messages
      */
     uint8_t messageHash[MAX_DIGEST_LENGTH];
+
     /**
      * The s0
      */
@@ -627,9 +690,39 @@ private:
     bool multiStreamAvailable;
 
     /**
-     * True if PBX enrollment is enabled.
+     * Enable MitM (PBX) enrollment
+     * 
+     * If set to true then ZRTP honors the PBX enrollment flag in
+     * Commit packets and calls the appropriate user callback
+     * methods. If the parameter is set to false ZRTP ignores the PBX
+     * enrollment flags.
      */
-    bool PBXEnrollment;
+    bool enableMitmEnrollment;
+
+    /**
+     * True if a valid trusted MitM key of the other peer is available, i.e. enrolled.
+     */
+    bool peerIsEnrolled;
+
+    /**
+     * Set to true if the Hello packet contained the M-flag (MitM flag).
+     * We use this later to check some stuff for SAS Relay processing
+     */
+    bool mitmSeen;
+
+    /**
+     * Temporarily store computed pbxSecret, if user accepts enrollment then
+     * it will copied to our ZID record of the PBX (MitM)  
+     */
+    uint8_t* pbxSecretTmp;
+    uint8_t  pbxSecretTmpBuffer[MAX_DIGEST_LENGTH];
+
+    /**
+     * If true then we will set the enrollment flag (E) in the confirm
+     * packets. Set to true if the PBX enrollment service started this ZRTP 
+     * session. Can be set to true only if mitmMode is also true. 
+     */
+    bool enrollmentMode;
 
     /**
      * Configuration data which algorithms to use.
@@ -651,6 +744,8 @@ private:
     ZrtpPacketConfirm  zrtpConfirm1;
     ZrtpPacketConfirm  zrtpConfirm2;
     ZrtpPacketPingAck  zrtpPingAck;
+    ZrtpPacketSASrelay zrtpSasRelay;
+    ZrtpPacketRelayAck zrtpRelayAck;
 
     /**
      * Random IV data to encrypt the confirm data, 128 bit for AES
@@ -663,10 +758,47 @@ private:
     /**
      * Variables to store signature data. Includes the signature type block
      */
-    uint8_t* signatureData;       // will be allocated when needed
+    const uint8_t* signatureData;       // will be set when needed
     int32_t  signatureLength;     // overall length in bytes
 
+    /**
+     * Is true if the other peer signaled SAS signature support in its Hello packet.
+     */
+    bool signSasSeen;
+
     uint32_t peerSSRC;            // peer's SSRC, required to setup PingAck packet
+
+    /**
+     * Enable or disable paranoid mode.
+     *
+     * The Paranoid mode controls the behaviour and handling of the SAS verify flag. If
+     * Panaoid mode is set to flase then ZRtp applies the normal handling. If Paranoid
+     * mode is set to true then the handling is:
+     *
+     * <ul>
+     * <li> Force the SAS verify flag to be false at srtpSecretsOn() callback. This gives
+     *      the user interface (UI) the indication to handle the SAS as <b>not verified</b>.
+     *      See implementation note below.</li>
+     * <li> Don't set the SAS verify flag in the <code>Confirm</code> packets, thus the other
+     *      also must report the SAS as <b>not verified</b>.</li>
+     * <li> ignore the <code>SASVerified()</code> function, thus do not set the SAS to verified
+     *      in the ZRTP cache. </li>
+     * <li> Disable the <b>Trusted PBX MitM</b> feature. Just send the <code>SASRelay</code> packet
+     *      but do not process the relayed data. This protects the user from a malicious
+     *      "trusted PBX".</li>
+     * </ul>
+     * ZRtp performs alls other steps during the ZRTP negotiations as usual, in particular it
+     * computes, compares, uses, and stores the retained secrets. This avoids unnecessary warning
+     * messages. The user may enable or disable the Paranoid mode on a call-by-call basis without
+     * breaking the key continuity data.
+     *
+     * <b>Implementation note:</b></br>
+     * An application shall always display the SAS code if the SAS verify flag is <code>false</code>.
+     * The application shall also use mechanisms to remind the user to compare the SAS code, for
+     * example useing larger fonts, different colours and other display features.
+     */
+    bool paranoidMode;
+
     /**
      * Find the best Hash algorithm that is offered in Hello.
      *
@@ -757,6 +889,11 @@ private:
      */
     bool checkMultiStream(ZrtpPacketHello* hello);
 
+    /**
+     * Save the computed MitM secret to the ZID record of the peer
+     */
+    void writeEnrollmentPBX();
+
     /**
      * Compute my hvi value according to ZRTP specification.
      */
@@ -775,6 +912,8 @@ private:
 
     void generateKeysMultiStream();
 
+    void computePBXSecret();
+
     void setNegotiatedHash(AlgorithmEnum* hash);
 
     /*
@@ -978,6 +1117,14 @@ private:
      */
     ZrtpPacketPingAck* preparePingAck(ZrtpPacketPing* ppkt);
 
+    /**
+     * Prepare the RelayAck packet.
+     *
+     * This method prepares the RelayAck packet. The input to this method is the
+     * SASrelay packet received from the peer.
+     */
+    ZrtpPacketRelayAck* prepareRelayAck(ZrtpPacketSASrelay* srly, uint32_t* errMsg);
+    
     /**
      * Prepare a GoClearAck packet w/o HMAC
      *
