@@ -22,6 +22,8 @@
 #ifndef _AESCFB_H__
 #define _AESCFB_H__
 
+#include <stdint.h>
+
 /**
  * @file aesCFB.h
  * @brief Function that provide AES CFB mode support
@@ -54,11 +56,8 @@
  *    Length of the data in bytes
  */
 
-void aesCfbEncrypt(unsigned char *key,
-            unsigned int keyLength,
-            unsigned char* IV,
-            unsigned char *data,
-            unsigned int dataLength);
+void aesCfbEncrypt(uint8_t* key, int32_t keyLength, uint8_t* IV, uint8_t *data,
+                   int32_t dataLength);
 
 /**
  * Decrypt data with AES CFB mode, full block feedback size.
@@ -80,11 +79,8 @@ void aesCfbEncrypt(unsigned char *key,
  *    Length of the data in bytes
  */
 
-void aesCfbDecrypt(unsigned char *key,
-            unsigned int keyLength,
-            unsigned char* IV,
-            unsigned char *data,
-            unsigned int dataLength);
+void aesCfbDecrypt(uint8_t* key, int32_t keyLength, const uint8_t* IV, uint8_t *data,
+                   int32_t dataLength);
 /**
  * @}
  */
