@@ -18,16 +18,19 @@
 
 #include <stdio.h>
 #include <openssl/evp.h>
-#include <config.h>
+#include <libzrtpcpp-config.h>
 
-#ifdef _MSWINDOWS_
+#if defined(_MSC_VER) || defined(WIN32) || defined(_WIN32)
+#undef  _MSWINDOWS_
+#define _MSWINDOWS_
 #include <windows.h>
 #endif
+
 #if defined SOLARIS && !defined HAVE_PTHREAD_H
 #include <synch.h>
 #include <thread.h>
 #endif
-#if !defined _MSWINDOWS_ && !defined SOLARIS
+#if !defined(_MSWINDOWS_) && !defined SOLARIS
 #include <pthread.h>
 #endif
 
@@ -71,6 +74,7 @@ int finalizeOpenSSL ()
 }
 
 #ifdef _MSWINDOWS_
+#define __LOCKING
 
 static HANDLE *lock_cs;
 
@@ -108,7 +112,8 @@ static void myLockingCallback(int mode, int type, const char *file, int line) {
 #endif /* OPENSSL_SYS_WIN32 */
 
 
-#if defined SOLARIS && !defined HAVE_PTHREAD_H
+#if defined SOLARIS && !defined HAVE_PTHREAD_H && !defined(_MSWINDOWS)
+#define __LOCKING
 
 static mutex_t *lock_cs;
 static long *lock_count;
@@ -175,6 +180,7 @@ static unsigned long solaris_thread_id(void) {
 #endif /* SOLARIS */
 
 
+#ifndef __LOCKING
 static pthread_mutex_t* lock_cs;
 static long* lock_count;
 
@@ -233,4 +239,4 @@ static unsigned long pthreads_thread_id(void)
     return(ret);
 }
 */
-
+#endif
